/* Copyright (c) 2014 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
 *
 * @defgroup ble_sdk_app_template_main main.c
 * @{
 * @ingroup ble_sdk_app_template
 * @brief Template project main file.
 *
 * This file contains a template for creating a new application. It has the code necessary to wakeup
 * from button, advertise, get a connection restart advertising on disconnect and if no new
 * connection created go back to system-off mode.
 * It can easily be used as a starting point for creating a new application, the comments identified
 * with 'YOUR_JOB' indicates where and how you can customize.
 */

#include <stdbool.h>
#include <stdint.h>
#include <string.h>

#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_params.h"
#include "boards.h"
#include "softdevice_handler.h"
#include "app_timer.h"
#include "fstorage.h"
#include "fds.h"
#include "peer_manager.h"

#include "bsp.h"
#include "bsp_btn_ble.h"
#include "sensorsim.h"
#include "nrf_gpio.h"
#include "ble_hci.h"
#include "ble_advdata.h"
#include "ble_advertising.h"
#include "ble_conn_state.h"
#include "BME280.h"
#include "BME280.c"
#include "BMI.c"
#include "BMI.h"
#include "nrf_nvic.h"
#include "ble_radio_notification.h"
#include "math.h"
#include "nrf_drv_gpiote.h"
//#include "app_simple_timer.h"
#include "fds.h"
#include "fstorage.h"
#include "OPT.c"
#include "OPT.h"
#include "ble_nus.h"

#include "ble_dfu.h"

//#define NRF_LOG_MODULE_NAME "APP"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"

#define IS_SRVC_CHANGED_CHARACT_PRESENT 1                                           /**< Include or not the service_changed characteristic. if not enabled, the server's database cannot be changed for the lifetime of the device*/

#define BLE_UUID_CUSTOM_SERVICE          0x1110
#define BLE_UUID_CUSTOM_CHAR1            0x1111
#define BLE_UUID_CUSTOM_CHAR2            0x1112
#define BLE_UUID_CUSTOM_CHAR3            0x1113
#define CHARACTERISTIC_SIZE 			 20
#define BLE_UUID_CUSTOM_DATA             0xBBBB;

#define DATA_SEND_IRQn									LPCOMP_IRQn
#define BLE_NUS_DATA_CHAR_LEN           20                                           /**< Maximum length of a data packet. */
#define UART_TX_BUF_SIZE                1024                                        /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE                1024                                        /**< UART RX buffer size. */

static uint16_t                 service_handle;
static ble_gatts_char_handles_t char1_handles;
static ble_gatts_char_handles_t char2_handles;

uint8_t channel_37 = 0;
uint8_t channel_38 = 0;
uint8_t channel_39 = 0;
uint16_t FILE_ID   =  0x0005;
uint16_t REC_KEY   =  0x0001;
uint16_t SAVE_ID = 0x0001;
uint16_t SAVE_KEY = 0x0001;
uint16_t System_ID = 0x0002;
uint16_t System_KEY = 0x0001;
uint16_t Setting_ID = 0x0003;
uint16_t Setting_KEY = 0x0001;
uint16_t DefaultFile_Settings = 0x0004;
uint16_t DefaultRec_Settings = 0x0001;
uint16_t j  = 0x0000;

uint16_t file_read ;  //= 0x0005;
uint16_t rec_read ; // =  0x0001;

uint16_t iIndex = 0;
bool SystemOn = false;
bool Savesettings = false;
bool turnoffdisconnect = false;
	
static bool m_is_notifying_enabled = true;                     
uint8_t tx_power_level = 4;
static volatile bool ButtonPressed = false;
static volatile uint8_t ButtonLog = 0;

volatile static bool INTLIGHT = false;
static volatile uint8_t write_flag = 0;
int16_t ambh_th = 90;//
int16_t ambl_th = 0;
int16_t tmph_th = 99;//50
int16_t tmpl_th = 0;
bool light_int = 0;

uint32_t VALUEON =  10000;
uint8_t VALUEOFF = 0x1E;
uint32_t DataloggingOn = 12000;
float avg_i = 0.0;
float ratio = 0.0,ratio_1 = 5.23, ratio_5 = 1.44, ratio_10 = 1, ratio_30_60 = 0.6;
//int weeks = 0,change_weeks = 0,change_weeks_1 = 0,week_30_60 = 409, ad_week = 0 ,weeks_flash = 0;
uint16_t weeks = 0,change_weeks = 0,change_weeks_1 = 0,week_30_60 = 409, ad_week = 0 ,weeks_flash = 0;

int16_t count_week = 0;
bool board_clean = 1;
uint8_t tamp_th = 60;
uint8_t data_counter = 0;
bool send_data = false;
bool send_ID =  false;
bool TurnOff = false;
bool deletedata = false;
bool DefaultSettings = false;
bool startdatalogging = false;
#define INTERVAL     APP_TIMER_TICKS(VALUEON, APP_TIMER_PRESCALER) //10sec on
#define INTERVAL1     APP_TIMER_TICKS(VALUEOFF*1000, APP_TIMER_PRESCALER)  //20sec off

uint16_t da[15];
int ONOFF[10];
APP_TIMER_DEF(m_a_timer_id);
APP_TIMER_DEF(m_b_timer_id);
uint8_t s=0;
#ifdef BSP_BUTTON_0
    #define PIN_IN ARDUINO_8_PIN
#endif
#ifndef PIN_IN
    #error "Please indicate input pin"
#endif



#if (NRF_SD_BLE_API_VERSION == 3)
#define NRF_BLE_MAX_MTU_SIZE            GATT_MTU_SIZE_DEFAULT                       /**< MTU size used in the softdevice enabling and to reply to a BLE_GATTS_EVT_EXCHANGE_MTU_REQUEST event. */
#endif

#define APP_FEATURE_NOT_SUPPORTED       BLE_GATT_STATUS_ATTERR_APP_BEGIN + 2        /**< Reply when unsupported features are requested. */

#define CENTRAL_LINK_COUNT              0                                           /**< Number of central links used by the application. When changing this number remember to adjust the RAM settings*/
#define PERIPHERAL_LINK_COUNT           1                                           /**< Number of peripheral links used by the application. When changing this number remember to adjust the RAM settings*/
#define APP_CFG_NON_CONN_ADV_TIMEOUT    0   
//#define DEVICE_NAME                     "ROAMBEE-BEECON"                           /**< Name of device. Will be included in the advertising data. */
#define MANUFACTURER_NAME               "NordicSemiconductor"                       /**< Manufacturer. Will be passed to Device Information Service. */
#define APP_ADV_INTERVALFAST                800                                         /**< The advertising interval (in units of 0.625 ms. This value corresponds to 187.5 ms). */
#define APP_ADV_INTERVALSLOW                16000                                         /**< The advertising interval (in units of 0.625 ms. This value corresponds to 187.5 ms). */


#define APP_ADV_INTERVAL                2000//1000                                         /**< The advertising interval (in units of 0.625 ms. This value corresponds to 187.5 ms). */
#define APP_ADV_TIMEOUT_IN_SECONDS      1//180                                         /**< The advertising timeout in units of seconds. */

#define APP_ADV_2_INTERVAL               160                                          /**< Alternate advertising interval (in units of 0.625 ms. This value corresponds to 50 ms). */
#define APP_ADV_2_TIMEOUT_IN_SECONDS     15//15                                          /**< Alternate advertising timeout in units of seconds. */

#define APP_TIMER_PRESCALER             0                                           /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_OP_QUEUE_SIZE         10 //4                                           /**< Size of timer operation queues. */

#define HI_UINT8(a) (((a) >> 4) & 0x0F)
#define LO_UINT8(a) ((a) & 0x0F)

#define MIN_CONN_INTERVAL               MSEC_TO_UNITS(400, UNIT_1_25_MS)            /**< Minimum acceptable connection interval (0.1 seconds). */
#define MAX_CONN_INTERVAL               MSEC_TO_UNITS(500, UNIT_1_25_MS)            /**< Maximum acceptable connection interval (0.2 second). */
#define SLAVE_LATENCY                   0                                           /**< Slave latency. */
#define CONN_SUP_TIMEOUT                MSEC_TO_UNITS(8000, UNIT_10_MS)             /**< Connection supervisory timeout (4 seconds). */

//#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(100, APP_TIMER_PRESCALER)  /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
//#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(100, APP_TIMER_PRESCALER) /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000, APP_TIMER_PRESCALER)  /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000, APP_TIMER_PRESCALER) /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */

//#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(100, APP_TIMER_PRESCALER)  /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
//#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(100, APP_TIMER_PRESCALER) /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */

#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                           /**< Number of attempts before giving up the connection parameter negotiation. */

#define SEC_PARAM_BOND                  1                                           /**< Perform bonding. */
#define SEC_PARAM_MITM                  0                                           /**< Man In The Middle protection not required. */
#define SEC_PARAM_LESC                 0                                           /**< LE Secure Connections not enabled. */
#define SEC_PARAM_KEYPRESS              0                                           /**< Keypress notifications not enabled. */
#define SEC_PARAM_IO_CAPABILITIES       BLE_GAP_IO_CAPS_NONE                        /**< No I/O capabilities. */
#define SEC_PARAM_OOB                   0                                           /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE          7                                           /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE          16                                          /**< Maximum encryption key size. */

#define DEAD_BEEF                       0xDEADBEEF                                  /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */


static uint16_t m_conn_handle = BLE_CONN_HANDLE_INVALID;                            /**< Handle of the current connection. */

static ble_dfu_t m_dfus;                                                            /**< Structure used to identify the DFU service. */
static ble_gap_addr_t ble_addr;
	
float readFloatPressure( void );
float readTempC( void );
float readFloatHumidity( void );
void cal_weeks(uint32_t prf,int prf1);
void cal_count(int prf);
void fls_count(void);
uint16_t count_fls = 0;
uint8_t dl_interval = 1;	//In Minutes;
uint8_t battery = 0x00;

static volatile bool m_update_adv_pkts = false;

// YOUR_JOB: Use UUIDs for service(s) used in your application.
static ble_uuid_t m_adv_uuids[] = {{BLE_UUID_DEVICE_INFORMATION_SERVICE, BLE_UUID_TYPE_BLE,}}; /**< Universally unique service identifiers. */
static void advertising_start(uint8_t channel_37, uint8_t channel_38, uint8_t channel_39);
static void advertising_init(bool alt_data);

static void ble_dfu_evt_handler(ble_dfu_t * p_dfu, ble_dfu_evt_t * p_evt)
	
{
    switch (p_evt->type)
    {
        case BLE_DFU_EVT_INDICATION_DISABLED:
            NRF_LOG_INFO("Indication for BLE_DFU is disabled\r\n");
            break;

        case BLE_DFU_EVT_INDICATION_ENABLED:
            NRF_LOG_INFO("Indication for BLE_DFU is enabled\r\n");
            break;

        case BLE_DFU_EVT_ENTERING_BOOTLOADER:
            NRF_LOG_INFO("Device is entering bootloader mode!\r\n");
            break;
        default:
            NRF_LOG_INFO("Unknown event from ble_dfu\r\n");
            break;
    }
}


/**@brief Callback function for asserts in the SoftDevice.
 *
 * @details This function will be called in case of an assert in the SoftDevice.
 *
 * @warning This handler is an example only and does not fit a final product. You need to analyze
 *          how your product is supposed to react in case of Assert.
 * @warning On assert from the SoftDevice, the system can only recover on reset.
 *
 * @param[in] line_num   Line number of the failing ASSERT call.
 * @param[in] file_name  File name of the failing ASSERT call.
 */
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}


/**@brief Function for handling File Data Storage events.
 *
 * @param[in] p_evt  Peer Manager event.
 * @param[in] cmd
 */
static void fds_evt_handler(fds_evt_t const * const p_evt)
{
    if (p_evt->id == FDS_EVT_GC)
    {
        NRF_LOG_DEBUG("GC completed\n");
    }
}


/**@brief Function for handling Peer Manager events.
 *
 * @param[in] p_evt  Peer Manager event.
 */
static void pm_evt_handler(pm_evt_t const * p_evt)
{
    ret_code_t err_code;

    switch (p_evt->evt_id)
    {
        case PM_EVT_BONDED_PEER_CONNECTED:
        {
            NRF_LOG_DEBUG("Connected to previously bonded device\r\n");
            err_code = pm_peer_rank_highest(p_evt->peer_id);
            if (err_code != NRF_ERROR_BUSY)
            {
              APP_ERROR_CHECK(err_code);
							 // NRF_LOG_INFO("1 - %d \r\n", err_code);
            }
        } break; // PM_EVT_BONDED_PEER_CONNECTED

        case PM_EVT_CONN_SEC_START:
            break; // PM_EVT_CONN_SEC_START

        case PM_EVT_CONN_SEC_SUCCEEDED:
        {
            NRF_LOG_DEBUG("Link secured. Role: %d. conn_handle: %d, Procedure: %d\r\n",
                                 ble_conn_state_role(p_evt->conn_handle),
                                 p_evt->conn_handle,
                                 p_evt->params.conn_sec_succeeded.procedure);
            err_code = pm_peer_rank_highest(p_evt->peer_id);
            if (err_code != NRF_ERROR_BUSY)
            {
                APP_ERROR_CHECK(err_code);
							//  NRF_LOG_INFO("2 - %d \r\n", err_code);
            }
        } break; // PM_EVT_CONN_SEC_SUCCEEDED

        case PM_EVT_CONN_SEC_FAILED:
        {
            /** Often, when securing fails, it shouldn't be restarted, for security reasons.
             *  Other times, it can be restarted directly.
             *  Sometimes it can be restarted, but only after changing some Security Parameters.
             *  Sometimes, it cannot be restarted until the link is disconnected and reconnected.
             *  Sometimes it is impossible, to secure the link, or the peer device does not support it.
             *  How to handle this error is highly application dependent. */
        } break; // PM_EVT_CONN_SEC_FAILED

        case PM_EVT_CONN_SEC_CONFIG_REQ:
        {
            // Reject pairing request from an already bonded peer.
            pm_conn_sec_config_t conn_sec_config = {.allow_repairing = false};
            pm_conn_sec_config_reply(p_evt->conn_handle, &conn_sec_config);
        } break; // PM_EVT_CONN_SEC_CONFIG_REQ

        case PM_EVT_STORAGE_FULL:
        {
            // Run garbage collection on the flash.
            err_code = fds_gc();
            if (err_code == FDS_ERR_BUSY || err_code == FDS_ERR_NO_SPACE_IN_QUEUES)
            {
                // Retry.
            }
            else
            {
               APP_ERROR_CHECK(err_code);
							 // NRF_LOG_INFO("3 - %d \r\n", err_code);
            }
        } break; // PM_EVT_STORAGE_FULL

        case PM_EVT_ERROR_UNEXPECTED:
            // Assert.
            APP_ERROR_CHECK(p_evt->params.error_unexpected.error);
            break; // PM_EVT_ERROR_UNEXPECTED

        case PM_EVT_PEER_DATA_UPDATE_SUCCEEDED:
            break; // PM_EVT_PEER_DATA_UPDATE_SUCCEEDED

        case PM_EVT_PEER_DATA_UPDATE_FAILED:
            // Assert.
            APP_ERROR_CHECK_BOOL(false);
            break; // PM_EVT_PEER_DATA_UPDATE_FAILED

        case PM_EVT_PEER_DELETE_SUCCEEDED:
            break; // PM_EVT_PEER_DELETE_SUCCEEDED

        case PM_EVT_PEER_DELETE_FAILED:
            // iAssert.
            APP_ERROR_CHECK(p_evt->params.peer_delete_failed.error);
            break; // PM_EVT_PEER_DELETE_FAILED

        case PM_EVT_PEERS_DELETE_SUCCEEDED:
            advertising_start(channel_37,channel_38,channel_39);
            break; // PM_EVT_PEERS_DELETE_SUCCEEDED

        case PM_EVT_PEERS_DELETE_FAILED:
            // Assert.
            APP_ERROR_CHECK(p_evt->params.peers_delete_failed_evt.error);
            break; // PM_EVT_PEERS_DELETE_FAILED

        case PM_EVT_LOCAL_DB_CACHE_APPLIED:
            break; // PM_EVT_LOCAL_DB_CACHE_APPLIED

        case PM_EVT_LOCAL_DB_CACHE_APPLY_FAILED:
            // The local database has likely changed, send service changed indications.
            pm_local_database_has_changed();
            break; // PM_EVT_LOCAL_DB_CACHE_APPLY_FAILED

        case PM_EVT_SERVICE_CHANGED_IND_SENT:
            break; // PM_EVT_SERVICE_CHANGED_IND_SENT

        case PM_EVT_SERVICE_CHANGED_IND_CONFIRMED:
            break; // PM_EVT_SERVICE_CHANGED_IND_SENT

        default:
            // No implementation needed.
            break;
    }
}


static void my_fds_evt_handler(fds_evt_t const * const p_fds_evt)
{
    switch (p_fds_evt->id)
    {
        case FDS_EVT_INIT:
            if (p_fds_evt->result != FDS_SUCCESS)
            {
                // Initialization failed.
            }
            break;
				case FDS_EVT_WRITE:
						if (p_fds_evt->result == FDS_SUCCESS)
						{
							write_flag=1;
						}
						break;
        default:
            break;
    }
}

void split(int prs,int * x, int * y) 
{
		*x = prs  & 0xff;
//		nrf_delay_ms(50);
		*y = (prs>>8) & 0xff;
//		nrf_delay_ms(50);
		return;
}

void splitmem(uint16_t prs,uint8_t * x, uint8_t * y) 
{
		*x = prs  & 0xff;
//		nrf_delay_ms(50);
		*y = (prs>>8) & 0xff;
//		nrf_delay_ms(50);
		return;
}

static ret_code_t fds_test_write(uint16_t nGroupIndex, uint8_t ntemp_msb, uint8_t ntemp_lsb,uint8_t nhum_msb, uint8_t nhum_lsb, uint8_t nprs_msb, uint8_t nprs_lsb,  uint8_t nbattery, uint8_t nlight,uint16_t FID , uint16_t RID , uint8_t SaveData ,uint16_t LFID , uint16_t LRID,bool system_on_flag)
{
	uint8_t  m_deadbeef[10] = {'0','0','0','0','0','0','0','0','0','0'} ;//{0x02,0x03,0x00,0x00,0x00};
	uint8_t lfidmsb = 0,lfidlsb = 0;
	uint8_t rfidmsb = 0, rfidlsb = 0;
//	uint8_t ridmsb, ridlsb;
	uint8_t weekmsb = 0, weeklsb = 0;
	fds_record_t        record;
	fds_record_desc_t   record_desc;
	fds_record_chunk_t  record_chunk;

	if(SaveData == 1)
	{
		splitmem(LFID, &lfidlsb, &lfidmsb);
		splitmem(LRID, &rfidlsb, &rfidmsb);
		splitmem(nGroupIndex,&weeklsb,&weekmsb);
		m_deadbeef[0] = lfidmsb; //0x00
		m_deadbeef[1] = lfidlsb; //0x05
		m_deadbeef[2] = rfidmsb; //0x00
		m_deadbeef[3] = rfidlsb; //0x01
		m_deadbeef[4] = weekmsb;
		m_deadbeef[5] = weeklsb;
		m_deadbeef[6] = 0x00;
		m_deadbeef[7] = 0x00;
		m_deadbeef[8] = 0x00;
		m_deadbeef[9] = 0x00;
		
	}	
	else if(SaveData == 2)
	{		
		splitmem(nGroupIndex, &lfidlsb, &lfidmsb);
		m_deadbeef[0] = lfidmsb;
		m_deadbeef[1] = lfidlsb;
		m_deadbeef[2] = ntemp_msb;
		m_deadbeef[3] = ntemp_lsb;
		m_deadbeef[4] = nbattery;
		m_deadbeef[5] = nlight;
		m_deadbeef[6] = nhum_msb;
		m_deadbeef[7] = nhum_lsb;
		m_deadbeef[8] = nprs_msb;
		m_deadbeef[9] = nprs_lsb;		
	}
	else if(SaveData == 3)
	{		
		splitmem(weeks,&weeklsb,&weekmsb);
		m_deadbeef[0] = system_on_flag; //0x00
		m_deadbeef[1] = VALUEOFF; //0x05
		m_deadbeef[2] = weekmsb; //0x00
		m_deadbeef[3] = weeklsb; //0x01
		m_deadbeef[4] = 0x00;
		m_deadbeef[5] = 0x00;
		m_deadbeef[6] = 0x00;
		m_deadbeef[7] = 0x00;
		m_deadbeef[8] = 0x00;
		m_deadbeef[9] = 0x00;
	}
	else if(SaveData == 4)
	{		
		splitmem(LFID, &lfidlsb, &lfidmsb);
		splitmem(LRID, &rfidlsb, &rfidmsb);
		splitmem(nGroupIndex,&weeklsb,&weekmsb);
		m_deadbeef[0] = lfidmsb; //0x00
		NRF_LOG_INFO("lfidmsb %d \r\n",m_deadbeef[0]);
		m_deadbeef[1] = lfidlsb; //0x05
		NRF_LOG_INFO("lfillsb %d \r\n",m_deadbeef[1] );
		m_deadbeef[2] = rfidmsb; //0x00
		NRF_LOG_INFO("rfidmsb %d \r\n",m_deadbeef[2] );
		m_deadbeef[3] = rfidlsb; //0x01
		NRF_LOG_INFO("rfidlsb %d \r\n",m_deadbeef[3] );
		m_deadbeef[4] = weekmsb;
		NRF_LOG_INFO("weekmsb %d \r\n",m_deadbeef[4] );
		m_deadbeef[5] = weeklsb;
		NRF_LOG_INFO("weeklsb %d \r\n",m_deadbeef[5] );
		m_deadbeef[6] = 0x00;
		m_deadbeef[7] = 0x00;
		m_deadbeef[8] = 0x00;
		m_deadbeef[9] = 0x00;
	}
	
	else if(SaveData == 5)
	{		
		m_deadbeef[0] = VALUEOFF; //Valueoff
		m_deadbeef[1] = tx_power_level; //Tx_power
		m_deadbeef[2] = tamp_th; //Tamp_th
		m_deadbeef[3] = channel_37; //channel37
		m_deadbeef[4] = channel_38; //channel38
		m_deadbeef[5] = channel_39; //channel39
		m_deadbeef[6] = dl_interval; //interval					
	}


	// Set up data.
	record_chunk.p_data        = m_deadbeef;
	record_chunk.length_words   = 10;
	// Set up record.
	record.file_id              = FID;
	record.key              		= RID;
	record.data.p_chunks       = &record_chunk;
	record.data.num_chunks   = 1;
			
	ret_code_t ret = fds_record_write(&record_desc, &record);
	if (ret != FDS_SUCCESS)
	{
			return ret;
	}
	NRF_LOG_INFO("Writing Record ID = %d \r\n",record_desc.record_id);
	return NRF_SUCCESS;
}

/*

static ret_code_t fds_test_write(uint16_t nGroupIndex, uint8_t ntemp_msb, uint8_t ntemp_lsb, uint8_t nbattery, uint8_t nlight,uint16_t FID , uint16_t RID , uint8_t SaveData ,uint16_t LFID , uint16_t LRID,bool system_on_flag)
{
//	uint8_t  m_deadbeef[8] = {'0','0','0','0','0','0','0','0'} ;//{0x02,0x03,0x00,0x00,0x00};
	uint8_t  m_deadbeef[7] = {'0','0','0','0','0','0','0'} ;//{0x02,0x03,0x00,0x00,0x00};

	uint8_t lfidmsb = 0,lfidlsb = 0;
	uint8_t rfidmsb = 0, rfidlsb = 0;
//	uint8_t ridmsb, ridlsb;
	uint8_t weekmsb = 0, weeklsb = 0;
	fds_record_t        record;
	fds_record_desc_t   record_desc;
	fds_record_chunk_t  record_chunk;

	if(SaveData == 1)
	{
		splitmem(LFID, &lfidlsb, &lfidmsb);
		splitmem(LRID, &rfidlsb, &rfidmsb);
		splitmem(nGroupIndex,&weeklsb,&weekmsb);
		m_deadbeef[0] = lfidmsb; //0x00
		m_deadbeef[1] = lfidlsb; //0x05
		m_deadbeef[2] = rfidmsb; //0x00
		m_deadbeef[3] = rfidlsb; //0x01
		m_deadbeef[4] = weekmsb;
		m_deadbeef[5] = weeklsb;
		m_deadbeef[6] = 0x00;
	}	
	else if(SaveData == 2)
	{		
		splitmem(nGroupIndex, &lfidlsb, &lfidmsb);
		m_deadbeef[0] = lfidmsb;
		m_deadbeef[1] = lfidlsb;
		m_deadbeef[2] = ntemp_msb;
		m_deadbeef[3] = ntemp_lsb;
		m_deadbeef[4] = nbattery;
		m_deadbeef[5] = nlight;
		m_deadbeef[6] = 0x00;
	}
	else if(SaveData == 3)
	{		
		splitmem(weeks,&weeklsb,&weekmsb);
		m_deadbeef[0] = system_on_flag; //0x00
		m_deadbeef[1] = VALUEOFF; //0x05
		m_deadbeef[2] = weekmsb; //0x00
		m_deadbeef[3] = weeklsb; //0x01
		m_deadbeef[4] = 0x00;
		m_deadbeef[5] = 0x00;
		m_deadbeef[6] = 0x00;

	}
	else if(SaveData == 4)
	{		
		splitmem(LFID, &lfidlsb, &lfidmsb);
		splitmem(LRID, &rfidlsb, &rfidmsb);
		splitmem(nGroupIndex,&weeklsb,&weekmsb);
		m_deadbeef[0] = lfidmsb; //0x00
		m_deadbeef[1] = lfidlsb; //0x05
		m_deadbeef[2] = rfidmsb; //0x00
		m_deadbeef[3] = rfidlsb; //0x01
		m_deadbeef[4] = weekmsb;
		m_deadbeef[5] = weeklsb;
		m_deadbeef[6] = 0x00;

	}
	else if(SaveData == 5)
	{		
		m_deadbeef[0] = VALUEOFF; //Valueoff
		m_deadbeef[1] = tx_power_level; //Tx_power
		m_deadbeef[2] = tamp_th; //Tamp_th
		m_deadbeef[3] = channel_37; //channel37
		m_deadbeef[4] = channel_38; //channel38
		m_deadbeef[5] = channel_39; //channel39
		m_deadbeef[6] = dl_interval; //interval					
	}

	// Set up data.
	record_chunk.p_data        = m_deadbeef;
	record_chunk.length_words   = 7;
	// Set up record.
	record.file_id              = FID;
	record.key              		= RID;
	record.data.p_chunks       = &record_chunk;
	record.data.num_chunks   = 1;
			
	ret_code_t ret = fds_record_write(&record_desc, &record);
	if (ret != FDS_SUCCESS)
	{
			NRF_LOG_INFO("Write Fail\r\n");
		  NRF_LOG_INFO("ret error %d \r\n",ret);
			return ret;	
	}
	NRF_LOG_INFO("Writing Record ID = %d \r\n",record_desc.record_id);
	return NRF_SUCCESS;
}

*/
/*
static ret_code_t fds_read(uint16_t FID , uint16_t RID)
{
//		#define FILE_ID     0x1110
//		#define REC_KEY     0x2222
		fds_flash_record_t  flash_record;
		fds_record_desc_t   record_desc;
		fds_find_token_t    ftok ={0};//Important, make sure you zero init the ftok token
		uint8_t *data;
		uint32_t err_code;
		
		NRF_LOG_INFO("Start searching... \r\n");
		// Loop until all records with the given key and file ID have been found.
		while (fds_record_find(FID, RID, &record_desc, &ftok) == FDS_SUCCESS)
		{
				err_code = fds_record_open(&record_desc, &flash_record);
				if ( err_code != FDS_SUCCESS)
				{
					return err_code;		
				}
				
				NRF_LOG_INFO("Found Record ID = %d\r\n",record_desc.record_id);
				NRF_LOG_INFO("Data = ");
				data = (uint8_t *) flash_record.p_data;
				for (uint8_t i=0;i<flash_record.p_header->tl.length_words;i++)
				{
					NRF_LOG_INFO("%d ",data[i]);
				}
				NRF_LOG_INFO("\r\n");
				// Access the record through the flash_record structure.
				// Close the record when done.
				err_code = fds_record_close(&record_desc);
				if (err_code != FDS_SUCCESS)
				{
					return err_code;	
				}
		}
		return NRF_SUCCESS;
		
}

*/


static ret_code_t fds_test_find_and_delete (uint16_t FID)
{
	//  #define FILE     0x0003
	//	#define REC     0x001
		fds_record_desc_t   record_desc;
		fds_find_token_t    ftok;
	
		ftok.page=0;
		ftok.p_addr=NULL;
		// Loop and find records with same ID and rec key and mark them as deleted. 
		while (fds_record_find_in_file(FID,&record_desc, &ftok) == FDS_SUCCESS)
		{
			fds_record_delete(&record_desc);
			NRF_LOG_INFO("Deleted record ID: %d \r\n",record_desc.record_id);
		}
		// call the garbage collector to empty them, don't need to do this all the time, this is just for demonstration
		ret_code_t ret = fds_gc();
		if (ret != FDS_SUCCESS)
		{
			  NRF_LOG_INFO("ret error %d \r\n",ret);
				return ret;
		}
		return NRF_SUCCESS;
}



static ret_code_t fds_test_init (void)
{
	
		ret_code_t ret = fds_register(my_fds_evt_handler);
		if (ret != FDS_SUCCESS)
		{
					return ret;
				
		}
		ret = fds_init();
		if (ret != FDS_SUCCESS)
		{
				return ret;
		}
		
		return NRF_SUCCESS;
		
}


/**@brief Function for the GAP initialization.
 *
 * @details This function sets up all the necessary GAP (Generic Access Profile) parameters of the
 *          device including the device name, appearance, and the preferred connection parameters.
 */
static void gap_params_init(void)
{
		uint32_t                err_code;
		ble_gap_conn_params_t   gap_conn_params;
		ble_gap_conn_sec_mode_t sec_mode;
		uint8_t DEVICE_NAME[8];

		err_code = sd_ble_gap_addr_get(&ble_addr);
		APP_ERROR_CHECK(err_code);


		DEVICE_NAME[0] ='C';
		DEVICE_NAME[1] ='4';

		if((HI_UINT8(ble_addr.addr[2]) >= 0) && (HI_UINT8(ble_addr.addr[2]) <= 0x09))
		{
			DEVICE_NAME[2] = HI_UINT8(ble_addr.addr[2]) + '0';
		}
		else
		{
			DEVICE_NAME[2] = HI_UINT8(ble_addr.addr[2]) + '7';
		}

		if((LO_UINT8(ble_addr.addr[2]) >= 0) && (LO_UINT8(ble_addr.addr[2]) <= 0x09))
		{
			DEVICE_NAME[2+1] = LO_UINT8(ble_addr.addr[2]) + '0';
		}
		else
		{
			DEVICE_NAME[2+1] = LO_UINT8(ble_addr.addr[2]) + '7';
		}

		if((HI_UINT8(ble_addr.addr[1]) >= 0) && (HI_UINT8(ble_addr.addr[1]) <= 0x09))
		{
			DEVICE_NAME[4] = HI_UINT8(ble_addr.addr[1]) + '0';
		}
		else
		{
			DEVICE_NAME[4] = HI_UINT8(ble_addr.addr[1]) + '7';
		}

		if((LO_UINT8(ble_addr.addr[1]) >= 0) && (LO_UINT8(ble_addr.addr[1]) <= 0x09))
		{
			DEVICE_NAME[4+1] = LO_UINT8(ble_addr.addr[1]) + '0';
		}
		else
		{
			DEVICE_NAME[4+1] = LO_UINT8(ble_addr.addr[1]) + '7';
		}

		if((HI_UINT8(ble_addr.addr[0]) >= 0) && (HI_UINT8(ble_addr.addr[0]) <= 0x09))
		{
			DEVICE_NAME[6] = HI_UINT8(ble_addr.addr[0]) + '0';
		}
		else
		{
			DEVICE_NAME[6] = HI_UINT8(ble_addr.addr[0]) + '7';
		}

		if((LO_UINT8(ble_addr.addr[0]) >= 0) && (LO_UINT8(ble_addr.addr[0]) <= 0x09))
		{
			DEVICE_NAME[6+1] = LO_UINT8(ble_addr.addr[0]) + '0';
		}
		else
		{
			DEVICE_NAME[6+1] = LO_UINT8(ble_addr.addr[0]) + '7';
		}

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);


		err_code = sd_ble_gap_device_name_set(&sec_mode,
                                          (const uint8_t *)DEVICE_NAME,
                                          sizeof(DEVICE_NAME));
    APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("5 - %d \r\n", err_code);																			

    /* YOUR_JOB: Use an appearance value matching the application's use case.
       err_code = sd_ble_gap_appearance_set(BLE_APPEARANCE_);
     //APP_ERROR_CHECK(err_code); */
																					

    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
  //APP_ERROR_CHECK(err_code);
	//	NRF_LOG_INFO("6 - %d \r\n", err_code);																			
																					
		err_code = sd_ble_gap_tx_power_set(tx_power_level);
    APP_ERROR_CHECK(err_code);
    //NRF_LOG_INFO("7 % d \r\n", err_code);		
}

static uint32_t send_notification(/*uint8_t * data*/uint8_t * data)
{
    uint16_t len = 10;
    ble_gatts_hvx_params_t hvx_params;
    memset(&hvx_params, 0, sizeof(hvx_params));

    hvx_params.handle = char2_handles.value_handle;
    hvx_params.type   = BLE_GATT_HVX_NOTIFICATION;
    hvx_params.offset = 0;
    hvx_params.p_len  = &len;
    hvx_params.p_data = data;

    return sd_ble_gatts_hvx(m_conn_handle, &hvx_params);
}


/**@brief Function for initializing services that will be used by the application.
 */
static void services_init(void)
{
    uint32_t err_code;
    ble_dfu_init_t dfus_init;
	
	  ble_uuid_t ble_uuid;
    ble_srv_security_mode_t        dis_attr_md;
    uint8_t i=0;


    // Initialize the Device Firmware Update Service.
    memset(&dfus_init, 0, sizeof(dfus_init));

    dfus_init.evt_handler                               = ble_dfu_evt_handler;
    dfus_init.ctrl_point_security_req_write_perm        = SEC_SIGNED;
    dfus_init.ctrl_point_security_req_cccd_write_perm   = SEC_SIGNED;

    err_code = ble_dfu_init(&m_dfus, &dfus_init);
    APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("8 - %d \r\n", err_code);
	
	  /*create custom service*/
    BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_CUSTOM_SERVICE);

    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &ble_uuid, &service_handle);
    APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("Service Init: 9 - %d\r\n", err_code);
    //*create custom characteristics*/
    ble_gatts_char_md_t char_md;
    ble_gatts_attr_t    attr_char_value;
    ble_gatts_attr_md_t attr_md;
		ble_gatts_attr_md_t cccd_md;
    uint8_t initial_char_values[CHARACTERISTIC_SIZE];
    for(i=0;i<CHARACTERISTIC_SIZE;i++) initial_char_values[i] = i;

    /*char1 will be for writing*/
    memset(&char_md, 0, sizeof(char_md));

    char_md.char_props.write_wo_resp    = 1;
    char_md.char_props.write    = 1;
    char_md.p_char_user_desc = NULL;
    char_md.p_char_pf        = NULL;
    char_md.p_user_desc_md   = NULL;
    char_md.p_cccd_md        = NULL;
    char_md.p_sccd_md        = NULL;

    BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_CUSTOM_CHAR1);

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&dis_attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&dis_attr_md.write_perm);
    attr_md.read_perm  = dis_attr_md.read_perm;
    attr_md.write_perm = dis_attr_md.write_perm;
    attr_md.vloc       = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth    = 0;
    attr_md.wr_auth    = 0;
    attr_md.vlen       = 0;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid    = &ble_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = CHARACTERISTIC_SIZE;
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = CHARACTERISTIC_SIZE;
    attr_char_value.p_value   = initial_char_values;

    err_code =  sd_ble_gatts_characteristic_add(service_handle, &char_md, &attr_char_value, &char1_handles);
    APP_ERROR_CHECK(err_code);
	//	NRF_LOG_INFO("Service: 10 - %d\r\n", err_code);


    /*Custom characterstic 2*/
    BLE_UUID_BLE_ASSIGN(ble_uuid, BLE_UUID_CUSTOM_DATA);

    memset(&cccd_md, 0, sizeof(cccd_md));
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cccd_md.write_perm);
    cccd_md.vloc = BLE_GATTS_VLOC_STACK;

    memset(&char_md, 0, sizeof(char_md));
		char_md.char_props.write_wo_resp    = 1;
    char_md.char_props.write    = 1;
    char_md.char_props.notify = 1;
    char_md.p_char_user_desc  = NULL;
    char_md.p_char_pf         = NULL;
    char_md.p_user_desc_md    = NULL;
    char_md.p_cccd_md         = &cccd_md;
    char_md.p_sccd_md         = NULL;

    memset(&attr_md, 0, sizeof(attr_md));

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&attr_md.write_perm);
    attr_md.read_perm  = dis_attr_md.read_perm;
    attr_md.write_perm = dis_attr_md.write_perm;
    attr_md.vloc    = BLE_GATTS_VLOC_STACK;
    attr_md.rd_auth = 0;
    attr_md.wr_auth = 0;
    attr_md.vlen    = 1;

    memset(&attr_char_value, 0, sizeof(attr_char_value));

    attr_char_value.p_uuid    = &ble_uuid;
    attr_char_value.p_attr_md = &attr_md;
    attr_char_value.init_len  = 20;
    attr_char_value.init_offs = 0;
    attr_char_value.max_len   = 20;

    err_code =  sd_ble_gatts_characteristic_add(service_handle, &char_md, &attr_char_value, &char2_handles);
    APP_ERROR_CHECK(err_code);
	//	NRF_LOG_INFO("11 - %d \r\n", err_code);
		NRF_LOG_INFO("Service %d\r\n", err_code);

    
}
void read_flash_data_conf(void)
{
		uint32_t err_code;
		uint8_t sensor_data[10];
	//  uint16_t sensor_data[10];
		fds_flash_record_t  flash_record;
		memset(sensor_data, 0, 10);
		fds_record_desc_t   record_desc;
		fds_find_token_t    ftok ={0};//Important, make sure you zero init the ftok token
		uint8_t *data;
	//		uint16_t *data;

		// Loop until all records with the given key and file ID have been found.
	//	while (fds_record_find_in_file(file_read,&record_desc, &ftok) == FDS_SUCCESS)
		while (fds_record_find(DefaultFile_Settings,DefaultRec_Settings,&record_desc, &ftok) == FDS_SUCCESS)	
		{
			err_code = fds_record_open(&record_desc, &flash_record);
			
			data = (uint8_t *) flash_record.p_data;
			for (uint8_t i=0;i<flash_record.p_header->tl.length_words;i++)
			{
				//NRF_LOG_INFO("Config Data: %d \r\n",data[i]);
				sensor_data[i] = data[i];
			}
			
			VALUEOFF = sensor_data[0];
			tx_power_level = sensor_data[1];	
			tamp_th = sensor_data[2];
			channel_37 = sensor_data[3];
			channel_38 = sensor_data[4];	
			channel_39 = sensor_data[5];	
			dl_interval = sensor_data[6];				

			// Access the record through the flash_record structure.
			// Close the record when done.
			err_code = fds_record_close(&record_desc);
			if (err_code == BLE_ERROR_NO_TX_PACKETS || err_code == NRF_ERROR_INVALID_STATE || err_code == BLE_ERROR_GATTS_SYS_ATTR_MISSING)  
			{
				APP_ERROR_CHECK(err_code);
			}
		}
}  

void read_flash_data(void)
{
		uint32_t err_code;
		uint8_t sensor_data[10];

		uint8_t weekmsb = 0, weeklsb = 0;
		fds_flash_record_t  flash_record;
		memset(sensor_data, 0, 10);
		fds_record_desc_t   record_desc;
		fds_find_token_t    ftok ={0};//Important, make sure you zero init the ftok token
		uint8_t *data;

		// Loop until all records with the given key and file ID have been found.
		while (fds_record_find(System_ID,System_KEY,&record_desc, &ftok) == FDS_SUCCESS)	
		{
			err_code = fds_record_open(&record_desc, &flash_record);
			
			data = (uint8_t *) flash_record.p_data;
			for (uint8_t i=0;i<flash_record.p_header->tl.length_words;i++)
			{
				//NRF_LOG_INFO("data - %d \r\n",data[i]);
				sensor_data[i] = data[i];
			}
			
			SystemOn = sensor_data[0];
			if (SystemOn == true)
			{
				VALUEOFF = sensor_data[1];
				
				weekmsb = sensor_data[2];	
				weeklsb = sensor_data[3];
				weeks = ((weekmsb) << 8 )| (weeklsb);	//record_read
			}
			else
				SystemOn = false;
				
			// Access the record through the flash_record structure.
			// Close the record when done.
			err_code = fds_record_close(&record_desc);
			if (err_code == BLE_ERROR_NO_TX_PACKETS || err_code == NRF_ERROR_INVALID_STATE || err_code == BLE_ERROR_GATTS_SYS_ATTR_MISSING)  
			{
					APP_ERROR_CHECK(err_code);
					//NRF_LOG_INFO("12 - %d \r\n", err_code);
			}
		}
}  
void read_flash_data_fds(void)
{
		uint32_t err_code;
		uint8_t lfidmsb,lfidlsb;
		uint8_t rfidmsb, rfidlsb;
		uint8_t sensor_data[10];
		uint8_t iCountMsb, iCountLsb;

		fds_flash_record_t  flash_record;
		memset(sensor_data, 0, 10);
		fds_record_desc_t   record_desc;
		fds_find_token_t    ftok ={0};//Important, make sure you zero init the ftok token
		uint8_t *data;
		while (fds_record_find(Setting_ID,Setting_KEY,&record_desc, &ftok) == FDS_SUCCESS)	
		{
			err_code = fds_record_open(&record_desc, &flash_record);
			
			data = (uint8_t *) flash_record.p_data;
			for (uint8_t i=0;i<flash_record.p_header->tl.length_words;i++)
			{
				//NRF_LOG_INFO("Read FLS: %d\r\n",data[i]);
				sensor_data[i] = data[i];
			}
			if (SystemOn == true)
			{
				lfidmsb = sensor_data[0];	
				lfidlsb = sensor_data[1];
				rfidmsb = sensor_data[2];	
				rfidlsb = sensor_data[3];
				iCountMsb = sensor_data[4];	
				iCountLsb = sensor_data[5];	
				FILE_ID = ((lfidmsb) << 8 )| (lfidlsb);	//record_read
				REC_KEY = ((rfidmsb) << 8 )| (rfidlsb);	//record_read
				iIndex = ((iCountMsb) << 8 )| (iCountLsb);	//record_read
				SAVE_KEY = FILE_ID - 4;
			}
			else
				SystemOn = false;
			

			// Access the record through the flash_record structure.
			// Close the record when done.
			err_code = fds_record_close(&record_desc);
			if (err_code == BLE_ERROR_NO_TX_PACKETS || err_code == NRF_ERROR_INVALID_STATE || err_code == BLE_ERROR_GATTS_SYS_ATTR_MISSING)  
			{
				APP_ERROR_CHECK(err_code);
				NRF_LOG_INFO("13 - %d \r\n", err_code);
			}
		}
}  



void data_send(void)
{
		uint32_t err_code;
		uint8_t sensor_data[14];
	//  uint16_t sensor_data[10];

		fds_flash_record_t  flash_record;
		memset(sensor_data, 0, 14);
		fds_record_desc_t   record_desc;
		fds_find_token_t    ftok ={0};//Important, make sure you zero init the ftok token
		uint8_t *data;
	//		uint16_t *data;

		// Loop until all records with the given key and file ID have been found.
	//	while (fds_record_find_in_file(file_read,&record_desc, &ftok) == FDS_SUCCESS)
		while (fds_record_find(file_read,rec_read,&record_desc, &ftok) == FDS_SUCCESS)	
		{
			err_code = fds_record_open(&record_desc, &flash_record);
			
			data = (uint8_t *) flash_record.p_data;
			for (uint8_t i=0;i<flash_record.p_header->tl.length_words;i++)
			{
				//NRF_LOG_INFO("Find Data: %d\r\n",data[i]);
				sensor_data[i] = data[i];
      //  NRF_LOG_INFO("%d ",sensor_data[i]);				
			}

			// Access the record through the flash_record structure.
			// Close the record when done.
			err_code = fds_record_close(&record_desc);
			NRF_LOG_INFO("%d\r\n",err_code);
		}

		err_code = send_notification(sensor_data);
		
		//NRF_LOG_INFO("Send_notif_error: %d\r\n",err_code);
		
	//	if (err_code == BLE_ERROR_NO_TX_PACKETS || err_code == NRF_ERROR_INVALID_STATE || err_code == BLE_ERROR_GATTS_SYS_ATTR_MISSING)  
	//	{
	//			APP_ERROR_CHECK(err_code)(err_code);
		     // NRF_LOG_INFO("%d \r\n", err_code);
	//	}

		

/*		data_counter++;
		if(data_counter == 100) 
		{
				send_data = false;
		}*/
}  


/**@brief Function for handling the Connection Parameters Module.
 *
 * @details This function will be called for all events in the Connection Parameters Module which
 *          are passed to the application.
 *          @note All this function does is to disconnect. This could have been done by simply
 *                setting the disconnect_on_fail config parameter, but instead we use the event
 *                handler mechanism to demonstrate its use.
 *
 * @param[in] p_evt  Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;

    if (p_evt->evt_type == BLE_CONN_PARAMS_EVT_FAILED)
    {
        err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
      APP_ERROR_CHECK(err_code);
			 // NRF_LOG_INFO("14 - %d \r\n", err_code);
    }
}


/**@brief Function for handling a Connection Parameters error.
 *
 * @param[in] nrf_error  Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}


/**@brief Function for initializing the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = BLE_GATT_HANDLE_INVALID;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;

    err_code = ble_conn_params_init(&cp_init);
  APP_ERROR_CHECK(err_code);
	 // NRF_LOG_INFO("15 - %d \r\n", err_code);
}


/**@brief Function for putting the chip into sleep mode.
 *
 * @note This function will not return.
 */
static void sleep_mode_enter(void)
{
    uint32_t err_code; //= bsp_indication_set(BSP_INDICATE_IDLE);
   //APP_ERROR_CHECK(err_code);
	 // NRF_LOG_INFO("16%d \r\n", err_code);

    // Prepare wakeup buttons.
    err_code = bsp_btn_ble_sleep_mode_prepare();
  APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("17 - %d \r\n", err_code);

    // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_power_system_off();
  APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("18 - %d \r\n", err_code);
}

// Timeout handler timer
static void timer_b_handler(void * p_context)
{
	if(nrf_gpio_pin_read(ARDUINO_1_PIN)==0) //if button is still pressed
	{  
			advertising_start(channel_37,channel_38,channel_39);
	}
}


// Timeout handler for the repeated timer
static void timer_a_handler(void * p_context)
{
	//nrf_gpio_pin_write(ARDUINO_6_PIN, 1);
//	nrf_gpio_pin_write(ARDUINO_8_PIN, 1);
	//advertising_init();
	//calibratebme280();
	advertising_start(channel_37,channel_38,channel_39);
}
// Create timers
static void create_timers()
{   
    uint32_t err_code;
    // Create timers
    err_code = app_timer_create(&m_a_timer_id,
                                APP_TIMER_MODE_SINGLE_SHOT,
                                timer_a_handler);
  APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("19 - %d \r\n", err_code);
	
	  err_code = app_timer_create(&m_b_timer_id,
                                APP_TIMER_MODE_SINGLE_SHOT,
                                timer_b_handler);
	
	
    APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("20 - %d \r\n", err_code);
}

/**@brief Function for handling advertising events.
 *
 * @details This function will be called for advertising events which are passed to the application.
 *
 * @param[in] ble_adv_evt  Advertising event.
 */
static void on_adv_evt(ble_adv_evt_t ble_adv_evt)
{
    uint32_t err_code;

    switch (ble_adv_evt)
    {
        case BLE_ADV_EVT_FAST:
//            err_code = bsp_indication_set(BSP_INDICATE_ADVERTISING);
  //        //APP_ERROR_CHECK(err_code);
            break;
        case BLE_ADV_EVT_IDLE:

						//nrf_gpio_pin_write(ARDUINO_6_PIN, 0);
				    //nrf_gpio_pin_write(ARDUINO_8_PIN, 0);
				    // *(uint32_t *)(0x40003000 + 0xFFC ) = 0;
            // *(uint32_t *)(0x40003000 + 0xFFC ) = 1;
				     
				    // *(uint32_t *)(0x40004000 + 0xFFC ) = 0;
            // *(uint32_t *)(0x40004000 + 0xFFC ) = 1;
						count_week = count_week - 1;
						if (count_week == 0)
						{
							cal_count(((VALUEOFF)));
							weeks = weeks - 1;
						}
						if(ButtonPressed == true)
						{
							ButtonPressed = false;
							err_code = sd_ble_gap_adv_stop();
							if (err_code != NRF_ERROR_INVALID_STATE)
							{
								APP_ERROR_CHECK(err_code);
								//NRF_LOG_INFO("21 - %d \r\n", err_code);
							}
							// modify advertising parameters
							advertising_init(false);
						}
						if(count_fls > 0)
						{
						 //nrf_delay_ms(200);
						 count_fls = count_fls - 1;
						 NRF_LOG_INFO("storing new data:%d\r\n",count_fls);
						}
						// Prepare wakeup buttons.
						err_code = app_timer_start(m_a_timer_id, APP_TIMER_TICKS((VALUEOFF*1000), APP_TIMER_PRESCALER), NULL);
						NRF_LOG_INFO("timererror %d\r\n", err_code);
						APP_ERROR_CHECK(err_code);
						//NRF_LOG_INFO("22 - %d \r\n", err_code);
            break;

        default:
            break;
    }
}

void change_1(int change_prf,int weeks,float ratio,uint16_t * change_week)
{
	if (change_prf <= 10) 
		*change_week = weeks/ratio;
	else if (change_prf >= 10)	  	  
		*change_week = weeks * ratio;
}
void change(int change_prf,int weeks,float ratio,uint16_t * change_week)
{
	if (change_prf >= 10) 
		*change_week = weeks/ratio;	   	   	   
	else if (change_prf <= 10)	  
		*change_week = weeks * ratio;
}

void Cal_week(float current,uint16_t *week) 
{	
	//*week = (battery/(current*168));// 168 = 24*7
	*week = (165000/(current*168));
	return;
}

void cal_weeks(uint32_t prf,int prf1)
{
	if (prf == 1)
	{
		avg_i = 19.0;
		ratio = ratio_1;
		Cal_week(avg_i ,&weeks);
		change(prf,weeks,ratio, &change_weeks);
	}
	
	if (prf == 5)
	{
		avg_i = 5.23;
		ratio = ratio_5;
		Cal_week(avg_i ,&weeks);
		change(prf,weeks,ratio, &change_weeks);
	}
	
	if ((prf == 30) || (prf == 10))
	{
		avg_i = 3.6;
		ratio = ratio_10;
		Cal_week(avg_i ,&weeks);
		change(prf,weeks,ratio, &change_weeks);
	}
	
	if ((prf == 60))
	{
		avg_i = 2.8;
		ratio = ratio_30_60;
		change(prf,week_30_60,ratio, &change_weeks);
	}
	
	if (prf1 == 1)
	{
		avg_i = 19.0;
		ratio = ratio_1;
		change_1(prf1,change_weeks,ratio, &change_weeks_1);
	}
	
	if (prf1 == 5)
	{
		avg_i = 5.23;
		ratio = ratio_5;
		change_1(prf1,change_weeks,ratio, &change_weeks_1);
	}
	if ((prf1 == 30) || (prf1 == 10))
	{
		avg_i = 3.6;
		ratio = ratio_10;
		change_1(prf1,change_weeks,ratio, &change_weeks_1);
	}
	if ((prf1 == 60))
	{
		avg_i = 2.8;
		ratio = ratio_30_60;
		change(prf1,change_weeks,ratio, &change_weeks_1);
	}
}


/**@brief Funtion that evaluates the value written in CCCD 
 *
 * @details This shall be called when there is a write event received from the stack. This 
 *          function will evaluate whether or not the peer has enabled notifications and
 *          start/stop notifications accordingly.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 */
static void on_writechar(ble_evt_t * p_ble_evt)
{
    ble_gatts_evt_write_t * p_evt_write = &p_ble_evt->evt.gatts_evt.params.write;

    if((p_evt_write->handle == char2_handles.cccd_handle)&&(p_evt_write->len == 2))
    {
        // CCCD written. Start notifications
        m_is_notifying_enabled = ble_srv_is_notification_enabled(p_evt_write->data);
				NRF_LOG_INFO("notifyerror: %d\r\n",m_is_notifying_enabled);

        if (m_is_notifying_enabled)
        {
            if(send_data == true)
						{
							send_data = false;
							data_send();
							/*if(send_ID == true)
							{
								datalog();
							}
							else
							{
								data_send();
							}*/
						}							
        }       
    }
}



/**@brief Function for handling the Application's BLE Stack events.
 *
 * @param[in] p_ble_evt  Bluetooth stack event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    uint32_t err_code = NRF_SUCCESS;
    uint8_t i=0;
	  
	
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_DISCONNECTED:
					   startdatalogging = false;
				     turnoffdisconnect = true;
					   NRF_LOG_INFO("DISCONNECTED.\r\n");
           // err_code = bsp_indication_set(BSP_INDICATE_IDLE);
           //APP_ERROR_CHECK(err_code);
			//			bsp_board_led_on(BSP_BOARD_LED_0);
			//			nrf_delay_ms(100);
			//			bsp_board_led_off(BSP_BOARD_LED_0);
            break; // BLE_GAP_EVT_DISCONNECTED

        case BLE_GAP_EVT_CONNECTED:
					   NRF_LOG_INFO("CONNECTED.\r\n");
				     startdatalogging = true;
        //    err_code = bsp_indication_set(BSP_INDICATE_CONNECTED);
        //  //APP_ERROR_CHECK(err_code);
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            break; // BLE_GAP_EVT_CONNECTED

        case BLE_GATTC_EVT_TIMEOUT:
            // Disconnect on GATT Client timeout event.
            NRF_LOG_DEBUG("GATT Client Timeout.\r\n");
            err_code = sd_ble_gap_disconnect(p_ble_evt->evt.gattc_evt.conn_handle,
                                             BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
          APP_ERROR_CHECK(err_code);
				   // NRF_LOG_INFO("23 - %d \r\n", err_code);
				
            break; // BLE_GATTC_EVT_TIMEOUT

        case BLE_GATTS_EVT_TIMEOUT:
          if (p_ble_evt->evt.gatts_evt.params.timeout.src == BLE_GATT_TIMEOUT_SRC_PROTOCOL)
            {
              //  err_code = sd_ble_gap_disconnect(m_conn_handle,
               //                                  BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
               //APP_ERROR_CHECK(err_code);
            }
            break;
				
				
        case BLE_EVT_USER_MEM_REQUEST:
            err_code = sd_ble_user_mem_reply(p_ble_evt->evt.gattc_evt.conn_handle, NULL);
          APP_ERROR_CHECK(err_code);
				    //NRF_LOG_INFO("24 - %d \r\n", err_code);
            break; // BLE_EVT_USER_MEM_REQUEST
				
					case BLE_GATTS_EVT_WRITE:
            NRF_LOG_INFO("BLE_GATTS_EVT_WRITE\r\n");
            for(i=0;i<p_ble_evt->evt.gatts_evt.params.write.len;i++) 
						NRF_LOG_INFO("BLE Write: %d\r\n",p_ble_evt->evt.gatts_evt.params.write.data[i]);
            da[0] = p_ble_evt->evt.gatts_evt.params.write.data[0];//prf configuration
					  NRF_LOG_INFO("da[0] %d \r\n", da[0]);
		        da[1] = p_ble_evt->evt.gatts_evt.params.write.data[1];//tx power
					  NRF_LOG_INFO("da[1] %d \r\n", da[1]);
						da[2] = p_ble_evt->evt.gatts_evt.params.write.data[2];//tamper_threshold
					  da[3] = p_ble_evt->evt.gatts_evt.params.write.data[3];//channel_config_37
					  da[4] = p_ble_evt->evt.gatts_evt.params.write.data[4];//channel_config_38
					  da[5] = p_ble_evt->evt.gatts_evt.params.write.data[5];//channel_config_39
				    da[6] = ((p_ble_evt->evt.gatts_evt.params.write.data[6]) << 8 )| (p_ble_evt->evt.gatts_evt.params.write.data[7]);//file_read
						da[7] = ((p_ble_evt->evt.gatts_evt.params.write.data[8]) << 8 )| (p_ble_evt->evt.gatts_evt.params.write.data[9]);//record_read
				    da[8] = p_ble_evt->evt.gatts_evt.params.write.data[10]; // dl_interval
				    da[9] = p_ble_evt->evt.gatts_evt.params.write.data[11]; // Switch_on_off
					  NRF_LOG_INFO("da[9] %d \r\n", da[9]);
						if(da[9] == 0x01)
						{
							
							TurnOff = true;
						}
						else if(da[9] == 0x02)
						{
								NRF_LOG_INFO("da[0]%d \r\n",da[0]);
								ONOFF[0] = da[0];// *1000;
							if (VALUEOFF != da[0])
								{
									change_weeks_1 = 0;
									cal_weeks(((VALUEOFF)),da[0]);
									weeks = change_weeks_1;
									count_week = 0;
								}
							
							
									tx_power_level= da[1];
									VALUEOFF = ONOFF[0];
									tamp_th = da[2];
									channel_37 = da[3];
									channel_38 = da[4];
									channel_39 = da[5];
									file_read    = da[6];
									rec_read    = da[7];
									dl_interval  = da[8];
									DefaultSettings = true; 
									send_data = true;
									send_ID = true;
									on_writechar(p_ble_evt);   
									da[9] = 0x03;
						}
						else if(da[9] == 0x03)				
						{
							file_read    = da[6];
							rec_read    = da[7];
							send_data = true;
							send_ID = true;
							on_writechar(p_ble_evt);			
						}
						
					else if(da[9] == 0x04)
					{
						 fds_test_find_and_delete(0x0001);
						
						 fds_test_find_and_delete(0x0003);
						 
						 NRF_LOG_INFO("FILEIDHEX %x\r\n", FILE_ID);
						
							for(j=FILE_ID;j>=0x0005;j--)
             { 							
							fds_test_find_and_delete(j);
							NRF_LOG_INFO("FILEHEX %d\r\n", j);	
							nrf_delay_ms(50); 
						 }
					//	 nrf_delay_ms(10000);
						 FILE_ID   =  0x0005;
						 REC_KEY   =  0x0001;
						 SAVE_ID = 0x0001;
						 SAVE_KEY = 0x0001;
						 Setting_ID = 0x0003;
						 Setting_KEY = 0x0001;

					//	 sd_nvic_SystemReset();
							//deletedata = true;
						bsp_board_led_on(BSP_BOARD_LED_0);
						nrf_delay_ms(100);
						bsp_board_led_off(BSP_BOARD_LED_0);
						
					}	
            break;
						

				case BLE_EVT_TX_COMPLETE:
						 data_send();
						//	nrf_delay_ms(4000);
					   break;

        case BLE_GATTS_EVT_RW_AUTHORIZE_REQUEST:
        {
            ble_gatts_evt_rw_authorize_request_t  req;
            ble_gatts_rw_authorize_reply_params_t auth_reply;

            req = p_ble_evt->evt.gatts_evt.params.authorize_request;

            if (req.type != BLE_GATTS_AUTHORIZE_TYPE_INVALID)
            {
                if ((req.request.write.op == BLE_GATTS_OP_PREP_WRITE_REQ)     ||
                    (req.request.write.op == BLE_GATTS_OP_EXEC_WRITE_REQ_NOW) ||
                    (req.request.write.op == BLE_GATTS_OP_EXEC_WRITE_REQ_CANCEL))
                {
                    if (req.type == BLE_GATTS_AUTHORIZE_TYPE_WRITE)
                    {
                        auth_reply.type = BLE_GATTS_AUTHORIZE_TYPE_WRITE;
                    }
                    else
                    {
                        auth_reply.type = BLE_GATTS_AUTHORIZE_TYPE_READ;
                    }
                    auth_reply.params.write.gatt_status = APP_FEATURE_NOT_SUPPORTED;
                    err_code = sd_ble_gatts_rw_authorize_reply(p_ble_evt->evt.gatts_evt.conn_handle,
                                                               &auth_reply);
                  APP_ERROR_CHECK(err_code);
										//NRF_LOG_INFO("25 - %d \r\n", err_code);
                }
            }
        } break; // BLE_GATTS_EVT_RW_AUTHORIZE_REQUEST

#if (NRF_SD_BLE_API_VERSION == 3)
        case BLE_GATTS_EVT_EXCHANGE_MTU_REQUEST:
            err_code = sd_ble_gatts_exchange_mtu_reply(p_ble_evt->evt.gatts_evt.conn_handle,
                                                       NRF_BLE_MAX_MTU_SIZE);
          APP_ERROR_CHECK(err_code);
				    //NRF_LOG_INFO("26 - %d \r\n", err_code);
            break; // BLE_GATTS_EVT_EXCHANGE_MTU_REQUEST
#endif

        default:
            // No implementation needed.
            break;
    }
}


/**@brief Function for dispatching a BLE stack event to all modules with a BLE stack event handler.
 *
 * @details This function is called from the BLE Stack event interrupt handler after a BLE stack
 *          event has been received.
 *
 * @param[in] p_ble_evt  Bluetooth stack event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
    /** The Connection state module has to be fed BLE events in order to function correctly
     * Remember to call ble_conn_state_on_ble_evt before calling any ble_conns_state_* functions. */
//  uint32_t err_code;  
	ble_conn_state_on_ble_evt(p_ble_evt);
    pm_on_ble_evt(p_ble_evt);
    ble_conn_params_on_ble_evt(p_ble_evt);
    bsp_btn_ble_on_ble_evt(p_ble_evt);
    on_ble_evt(p_ble_evt);
    ble_advertising_on_ble_evt(p_ble_evt);
    ble_dfu_on_ble_evt(&m_dfus, p_ble_evt);
    /*YOUR_JOB add calls to _on_ble_evt functions from each service your application is using
       ble_xxs_on_ble_evt(&m_xxs, p_ble_evt);
       ble_yys_on_ble_evt(&m_yys, p_ble_evt);
     */

}


/**@brief Function for dispatching a system event to interested modules.
 *
 * @details This function is called from the System event interrupt handler after a system
 *          event has been received.
 *
 * @param[in] sys_evt  System stack event.
 */
static void sys_evt_dispatch(uint32_t sys_evt)
{
    // Dispatch the system event to the fstorage module, where it will be
    // dispatched to the Flash Data Storage (FDS) module.
    fs_sys_event_handler(sys_evt);

    // Dispatch to the Advertising module last, since it will check if there are any
    // pending flash operations in fstorage. Let fstorage process system events first,
    // so that it can report correctly to the Advertising module.
    ble_advertising_on_sys_evt(sys_evt);
}


/**@brief Function for initializing the BLE stack.
 *
 * @details Initializes the SoftDevice and the BLE event interrupt.
 */
static void ble_stack_init(void)
{
    uint32_t err_code;

    nrf_clock_lf_cfg_t clock_lf_cfg = NRF_CLOCK_LFCLKSRC;

    // Initialize the SoftDevice handler module.
    SOFTDEVICE_HANDLER_INIT(&clock_lf_cfg, NULL);

    ble_enable_params_t ble_enable_params;
    err_code = softdevice_enable_get_default_config(CENTRAL_LINK_COUNT,
                                                    PERIPHERAL_LINK_COUNT,
                                                    &ble_enable_params);
  APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("27 - %d \r\n", err_code);

    // Check the ram settings against the used number of links
    CHECK_RAM_START_ADDR(CENTRAL_LINK_COUNT, PERIPHERAL_LINK_COUNT);

    // Enable BLE stack.
#if (NRF_SD_BLE_API_VERSION == 3)
    ble_enable_params.gatt_enable_params.att_mtu = NRF_BLE_MAX_MTU_SIZE;
#endif
    err_code = softdevice_enable(&ble_enable_params);
  APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("28 - %d \r\n", err_code);

    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
  APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("29 - %d \r\n", err_code);

    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_sys_evt_handler_set(sys_evt_dispatch);
  APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("30 - %d \r\n", err_code);
}


/**@brief Function for the Peer Manager initialization.
 *
 * @param[in] erase_bonds  Indicates whether bonding information should be cleared from
 *                         persistent storage during initialization of the Peer Manager.
 */
static void peer_manager_init(bool erase_bonds)
{
    ble_gap_sec_params_t sec_param;
    ret_code_t           err_code;

    err_code = pm_init();
  APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("31 - %d \r\n", err_code);

    if (erase_bonds)
    {
        err_code = pm_peers_delete();
      APP_ERROR_CHECK(err_code);
			  //NRF_LOG_INFO("32 - %d \r\n", err_code);
    }

    memset(&sec_param, 0, sizeof(ble_gap_sec_params_t));

    // Security parameters to be used for all security procedures.
    sec_param.bond           = SEC_PARAM_BOND;
    sec_param.mitm           = SEC_PARAM_MITM;
    sec_param.lesc           = SEC_PARAM_LESC;
    sec_param.keypress       = SEC_PARAM_KEYPRESS;
    sec_param.io_caps        = SEC_PARAM_IO_CAPABILITIES;
    sec_param.oob            = SEC_PARAM_OOB;
    sec_param.min_key_size   = SEC_PARAM_MIN_KEY_SIZE;
    sec_param.max_key_size   = SEC_PARAM_MAX_KEY_SIZE;
    sec_param.kdist_own.enc  = 1;
    sec_param.kdist_own.id   = 1;
    sec_param.kdist_peer.enc = 1;
    sec_param.kdist_peer.id  = 1;

    err_code = pm_sec_params_set(&sec_param);
  APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("33 - %d \r\n", err_code);

    err_code = pm_register(pm_evt_handler);
  APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("34 - %d \r\n", err_code);
		
		err_code = fds_register(fds_evt_handler);
  APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("35 - %d \r\n", err_code);
}
/**@brief Function for handling events from the BSP module.
 *
 * @param[in]   event   Event generated when button is pressed.
 */
static void bsp_event_handler(bsp_event_t event)
{
    uint32_t err_code;

    switch (event)
    {
			case BSP_EVENT_SLEEP:
				 //sleep_mode_enter();
					break; // BSP_EVENT_SLEEP

			case BSP_EVENT_DISCONNECT:
					err_code = sd_ble_gap_disconnect(m_conn_handle,
																					 BLE_HCI_REMOTE_USER_TERMINATED_CONNECTION);
					if (err_code != NRF_ERROR_INVALID_STATE)
					{
							APP_ERROR_CHECK(err_code);
						  //NRF_LOG_INFO("36 - %d \r\n", err_code);
					}
					break; // BSP_EVENT_DISCONNECT
					
			case BSP_EVENT_ADVERTISING_START:
				if(SystemOn == false)
				{
					SystemOn = true;
					ButtonLog = 2;
					//advertising_start();
					advertising_start(channel_37,channel_38,channel_39);					
					bsp_board_led_on(BSP_BOARD_LED_0);
					nrf_delay_ms(100);
					bsp_board_led_off(BSP_BOARD_LED_0);
				}						
			break;	
						
			case BSP_EVENT_KEY_1:
					if(SystemOn == true)
					{
						ButtonLog = 1;
					}
					break;
				case BSP_EVENT_KEY_0:
					if(SystemOn == true)
					{
						// stop BLE advertising
						ButtonPressed = true;
						bsp_board_led_on(BSP_BOARD_LED_1);
						nrf_delay_ms(100);
						bsp_board_led_off(BSP_BOARD_LED_1);
						// stop BLE advertising
						err_code = sd_ble_gap_adv_stop();
						if (err_code != NRF_ERROR_INVALID_STATE)
						{
								APP_ERROR_CHECK(err_code);
							 // NRF_LOG_INFO("37 - %d \r\n", err_code);
						}

						// modify advertising parameters
						advertising_init(true);

						// start BLE advertising
					//	err_code = ble_advertising_start(BLE_ADV_MODE_FAST);
						err_code = ble_advertising_start(BLE_ADV_MODE_FAST,channel_37,channel_38,channel_39);
						if (err_code != NRF_ERROR_INVALID_STATE)
						{
								APP_ERROR_CHECK(err_code);
							//NRF_LOG_INFO("38 - %d \r\n", err_code);
						}
					}
					break;
        default:
            break;
    }
}
/**@brief Function for initializing the Advertising functionality.
 */
static void advertising_init(bool alt_data)
{
		uint32_t               err_code=0;
    ble_advdata_t          advdata;
	  ble_advdata_t scanrsp;
	  unsigned int treal;
	  unsigned int tpoint;
	  uint8_t data=0;
	 uint8_t data1=0;
    ble_adv_modes_config_t options;
	  int p_msb,p_lsb, t_msb, t_lsb, h_msb, h_lsb;
	  float iTemperature = 0;
	  int32_t iHumidity = 0;	
  	int32_t iPressure = 0;
		nrf_delay_ms(2);
		iTemperature = readTempC();
	  nrf_delay_ms(2);
	  iPressure =  readFloatPressure();
		iPressure = (iPressure/100);
	  treal = (unsigned int) iTemperature;  
    tpoint = (unsigned int ) (iTemperature * 256); 
		nrf_delay_ms(2);
	  iPressure =  readFloatPressure();
		iPressure = (iPressure/100);
	  nrf_delay_ms(2);
		iHumidity = readFloatHumidity();
		nrf_delay_ms(2);
	  int p_value = iPressure;
	  nrf_delay_ms(2);
	//	light_sensor_init();
	 // nrf_delay_ms(2);
	//  ALS_sensor_init();
	  nrf_delay_ms(2);
//	  data = light_sensor_get_data(ALS_I2C_SLAVE_ADDR_B); //Top opt2
//	  nrf_delay_ms(2);
//	  data1 = light_sensor_get_data(ALS_I2C_SLAVE_ADDR_T);
//	  nrf_delay_ms(1000);	

    data = 0x02;//light_sensor_get_data(ALS_I2C_SLAVE_ADDR_B);
	  data1 = 0x03;//light_sensor_get_data(ALS_I2C_SLAVE_ADDR_T);
		if (data1 >= tamp_th)
			data = data | 0x80;
		else
			data = data | 0x00;	
		
    if(INTLIGHT)
		{
		  data = data | 0x80;
			INTLIGHT = false;
		}

		split(iHumidity, &h_lsb, &h_msb);
	  split(iTemperature, &t_lsb, &t_msb);
	  split(p_value,&p_lsb, &p_msb);
		nrf_delay_ms(2);
  // Variables used for manufacturer specific data
    ble_advdata_manuf_data_t adv_manuf_data;
    uint8_array_t            adv_manuf_data_array;
    uint8_t                  adv_manuf_data_data[15];
    // ---------------------------------------------
	   err_code = sd_ble_gap_addr_get(&ble_addr);
     APP_ERROR_CHECK(err_code); 
    // Build advertising data struct to pass into @ref ble_advertising_init.
    memset(&advdata, 0, sizeof(advdata));
		advdata.name_type               = BLE_ADVDATA_NO_NAME;
    advdata.include_appearance      = false;
    advdata.flags                   = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
    advdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
    advdata.uuids_complete.p_uuids  = m_adv_uuids;
		// Configuration of manufacturer specific data
    adv_manuf_data_data[0] = 0x52;
    adv_manuf_data_data[1] = 0x42;
		adv_manuf_data_data[2] = 0x04;
    adv_manuf_data_data[3] = 0x00;
    adv_manuf_data_data[4] = 0x04;													 
		adv_manuf_data_data[5] = treal;
		adv_manuf_data_data[6] = tpoint;
		adv_manuf_data_data[7] = 0x64;
		adv_manuf_data_data[8] = data;
		adv_manuf_data_data[9] = h_lsb;
		adv_manuf_data_data[10] = h_msb;
		adv_manuf_data_data[11] = p_msb;
		adv_manuf_data_data[12] = p_lsb;
		adv_manuf_data_data[13] = 0x00;
		adv_manuf_data_data[14] = 0x3E;
	  adv_manuf_data_array.p_data = adv_manuf_data_data;
		adv_manuf_data_array.size = 15;		
		adv_manuf_data.company_identifier = 0x4252;
		adv_manuf_data.data = adv_manuf_data_array;
		advdata.p_manuf_specific_data = &adv_manuf_data;
		memset(&scanrsp, 0, sizeof(scanrsp));
		scanrsp.name_type             = BLE_ADVDATA_FULL_NAME;						
		memset(&options, 0, sizeof(options));
		
		if (alt_data)
    {
			  options.ble_adv_fast_enabled  = true;
        // alternate advertising parameters
        options.ble_adv_fast_interval = APP_ADV_2_INTERVAL;
        options.ble_adv_fast_timeout  = APP_ADV_2_TIMEOUT_IN_SECONDS;
    }
    else
    {
			  options.ble_adv_fast_enabled  = true;
        // default advertising parameters
        options.ble_adv_fast_interval = APP_ADV_INTERVAL;
        options.ble_adv_fast_timeout  = APP_ADV_TIMEOUT_IN_SECONDS;
    }
		err_code = ble_advertising_init(&advdata,&scanrsp, &options, on_adv_evt, NULL);
		APP_ERROR_CHECK(err_code);
}



/**@brief Function for initializing buttons and leds.
 *
 * @param[out] p_erase_bonds  Will be true if the clear bonding button was pressed to wake the application up.
 */
static void buttons_leds_init(/*bool * p_erase_bonds*/)
{
    //bsp_event_t startup_event;
    uint32_t err_code = bsp_init(BSP_INIT_LED | BSP_INIT_BUTTONS,
                                 APP_TIMER_TICKS(100, APP_TIMER_PRESCALER),
                                 bsp_event_handler);
		APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("41 - %d \r\n", err_code);
	
    err_code = bsp_event_to_button_action_assign(0, BSP_BUTTON_ACTION_LONG_PUSH, BSP_EVENT_ADVERTISING_START);
    APP_ERROR_CHECK(err_code);
	
		err_code = bsp_event_to_button_action_assign(0, BSP_BUTTON_ACTION_LONG_PUSH_3SECS, BSP_EVENT_KEY_1);
    APP_ERROR_CHECK(err_code);
	 // NRF_LOG_INFO("42 - %d \r\n", err_code);
	
	//	err_code = bsp_event_to_button_action_assign(0, BSP_BUTTON_ACTION_PUSH, BSP_EVENT_WHITELIST_OFF); //add release action trigger on the button. call BSP_EVENT_KEY_4 event
	//	APP_ERROR_CHECK(err_code)(err_code);
}

/**@brief Function for the Power manager.
 */
static void power_manage(void)
{
		uint32_t err_code = sd_app_evt_wait();
  APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("43 - %d \r\n", err_code);
}

/**@brief Function for starting advertising.
 */
static void advertising_start(uint8_t channel_37, uint8_t channel_38, uint8_t channel_39)
{
   // uint32_t err_code == ble_advertising_start(BLE_ADV_MODE_FAST);
    uint32_t err_code = ble_advertising_start(BLE_ADV_MODE_FAST,channel_37,channel_38,channel_39);

    if (err_code != NRF_ERROR_INVALID_STATE)
    {
			 NRF_LOG_INFO("45 - %d \r\n", err_code);
			// APP_ERROR_CHECK(err_code);
						
			 err_code = ble_advertising_default(BLE_ADV_MODE_FAST);
			 if (err_code == NRF_ERROR_INVALID_PARAM)
			 {	 
				   ble_advertising_default(BLE_ADV_MODE_FAST);
					NRF_LOG_INFO("46 - %d \r\n", err_code); 
					APP_ERROR_CHECK(err_code);
			 }
      			
			   //err_code = 18 0x12 err_code = 07 0x07 
    }
}


/**@brief Radio notification event handler. */
static void on_radio_notification (bool radio_evt)
{
    if (radio_evt==false)
    {
        m_update_adv_pkts = true;
    }
}

/**@brief Function for initializing Radio Notification Software Interrupts.
 */
uint32_t radio_notification_init(uint32_t irq_priority, uint8_t notification_type, uint8_t notification_distance)
{
    return ble_radio_notification_init(irq_priority, notification_distance, on_radio_notification);
}

void fls_count(void)
{
	 //count_fls = 900/prf;
	 if(VALUEOFF == 0x01)
	 {
		count_fls = dl_interval * (60);
	 }
	 else if(VALUEOFF == 0x05)
	 {
		count_fls = dl_interval * (12);
	 }  
	 else if(VALUEOFF == 0x0A)
	 {
		count_fls = dl_interval* (5);
	 }
	 else if(VALUEOFF == 0x1E)
	 {
		count_fls = dl_interval* (2);
	 }
	 else if(VALUEOFF == 0x3C)
	 {
		count_fls = dl_interval* (1);
	 }
	 
	 NRF_LOG_INFO("count_fls %d\r\n", count_fls);
	 NRF_LOG_INFO("dl_interval %d\r\n", dl_interval);
}

/**@brief Update advertising data. */
void BME_sleep()
{
 write16(BME280_CTRL_MEAS_REG, 0x00);
}

static void update_adv_data()
{

	uint32_t  err_code=0;
	ble_advdata_t advdata;
	//ble_advdata_t scanrsp;
	int week1 = 0,week2 = 0;
	uint8_t data=0 ,data1 = 0;
	uint8_t battery = 0xff;
	unsigned int treal,tpoint;
	split(weeks,&week1,&week2);
	if (weeks > 255)
	battery = 0xff;
	else if (weeks <= 255)
	battery = weeks;
	float iTemperature = 0;
	int32_t iHumidity = 0;	
	float iPressure = 0;
	int p_msb,p_lsb, t_msb, t_lsb, h_msb, h_lsb;

	data = light_sensor_get_data(ALS_I2C_SLAVE_ADDR_B);
	data1 = light_sensor_get_data(ALS_I2C_SLAVE_ADDR_T);
	if (data1 >= tamp_th)
	data = data | 0x80;
	else
	data = data | 0x00;



	//!Variables used for manufacturer specific data
	ble_advdata_manuf_data_t adv_manuf_data;
	uint8_array_t            adv_manuf_data_array;
	uint8_t                  adv_manuf_data_data[15];
	//!---------------------------------------------
	err_code = sd_ble_gap_addr_get(&ble_addr);
	APP_ERROR_CHECK(err_code);
	//!Build advertising data struct to pass into @ref ble_advertising_init.
	memset(&advdata, 0, sizeof(advdata));
	advdata.name_type               =  BLE_ADVDATA_NO_NAME;
	advdata.include_appearance      = false;
	advdata.flags                   = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
	advdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
	advdata.uuids_complete.p_uuids  = m_adv_uuids;
	iTemperature = readTempC();
	treal = (unsigned int) iTemperature;  
	tpoint = (unsigned int ) (iTemperature * 256); 

	//nrf_delay_ms(100);
	iPressure = (unsigned int) readFloatPressure();
	iPressure = (unsigned int)(iPressure/100);
	//nrf_delay_ms(10);
	iHumidity = readFloatHumidity();
	// write_reset(BME280_RST_REG, 0xB6);
//	 write_reset(BME280_CTRL_MEAS_REG, 0x00);
 // write16(BME280_CTRL_MEAS_REG, 0x00);
	BME_sleep();
	
	

	//nrf_delay_ms(10);
	//int p_value = iPressure;
	//nrf_delay_ms(50);
	split(iHumidity, &h_lsb, &h_msb);
	split(iTemperature, &t_lsb, &t_msb);
	split(iPressure,&p_lsb, &p_msb);
		
	if(ButtonLog == 1)
		{
			NRF_LOG_INFO("Button:%d\r\n", ButtonLog);		
			bsp_board_led_on(BSP_BOARD_LED_1);
			nrf_delay_ms(100);
			bsp_board_led_off(BSP_BOARD_LED_1);
			ButtonLog = 0;
			
			if(!startdatalogging)
			{	
				REC_KEY = REC_KEY - 1;
				err_code=	fds_test_write(iIndex,'0','0','0','0','0','0','0','0',SAVE_ID,SAVE_KEY,1, FILE_ID, REC_KEY,'0');
				NRF_LOG_INFO("1 \r\n");
				APP_ERROR_CHECK(err_code);
				FILE_ID = FILE_ID + 0x0001;
				SAVE_KEY = SAVE_KEY + 0x0001;
				REC_KEY = 0x0001;
				NRF_LOG_INFO("FILE_ID %d\r\n",FILE_ID);
			//	err_code=fds_test_find_and_delete(Setting_ID);
		//		APP_ERROR_CHECK(err_code);
				nrf_delay_ms(50);
				NRF_LOG_INFO("weeks %d\r\n",weeks);
				err_code=fds_test_write(iIndex,'0','0','0','0','0','0','0','0',Setting_ID,Setting_KEY,4, FILE_ID, REC_KEY,'0');
				NRF_LOG_INFO("2 \r\n");	
				APP_ERROR_CHECK(err_code);
				nrf_delay_ms(50);
			}	
		}	
		
		if(ButtonLog == 2)
		{
			if(!startdatalogging)
			{	
				err_code=fds_test_write(0,'0','0','0','0','0','0','0','0',System_ID,System_KEY,3, '0', '0',SystemOn);
				NRF_LOG_INFO("3 \r\n");			
				APP_ERROR_CHECK(err_code);
				nrf_delay_ms(50);
				err_code = fds_test_write('0','0','0','0','0','0','0','0','0',DefaultFile_Settings,DefaultRec_Settings,5,'0','0','0');			
				NRF_LOG_INFO("4 \r\n");	
				APP_ERROR_CHECK(err_code);
				ButtonLog = 0;
			}	
		}
		
		if(count_fls == 0)
		{	
			NRF_LOG_INFO("Writing records %d \r\n",s);
			fls_count();
			s++;
			if(!startdatalogging)
			{	
				err_code =fds_test_write(iIndex,(uint8_t)tpoint, (uint8_t)treal,(uint8_t)h_lsb,(uint8_t)h_msb,(uint8_t)p_msb,(uint8_t)p_lsb, battery, data,FILE_ID,REC_KEY,2,0,0,0);
				NRF_LOG_INFO("6 \r\n");
				APP_ERROR_CHECK(err_code);
				NRF_LOG_INFO("N-%d \r\n", err_code);
				iIndex = iIndex + 1;
				REC_KEY = REC_KEY + 0x0001;
			//	err_code =fds_test_find_and_delete(Setting_ID);
			//	NRF_LOG_INFO("9 \r\n");
				APP_ERROR_CHECK(err_code);
				nrf_delay_ms(50);
				err_code =fds_test_write(iIndex,'0','0','0','0','0','0','0','0',Setting_ID,Setting_KEY,4, FILE_ID, REC_KEY,'0');
				NRF_LOG_INFO("7 %d \r\n", err_code);  
				APP_ERROR_CHECK(err_code);
				NRF_LOG_INFO("iIndex %d\r\n", iIndex);						
			}
		}
		
		if((TurnOff == true) && (turnoffdisconnect == true))
		{
			TurnOff = false;
			SystemOn = false;
			turnoffdisconnect = false;
			nrf_delay_ms(50);
		//	err_code =fds_test_find_and_delete(Setting_ID);
		//	NRF_LOG_INFO("11 \r\n");
			APP_ERROR_CHECK(err_code);
			err_code =fds_test_write(0,'0','0','0','0','0','0','0','0',System_ID,System_KEY,3, '0', '0',SystemOn);
			NRF_LOG_INFO("8-%d \r\n", err_code);
			APP_ERROR_CHECK(err_code);
			nrf_delay_ms(50);
			sleep_mode_enter();
		}
		
		if(DefaultSettings == true)
		{
			if(!startdatalogging)
			{	
				err_code =fds_test_write('0','0','0','0','0','0','0','0','0',DefaultFile_Settings,DefaultRec_Settings,5,'0','0','0');	
				NRF_LOG_INFO("Written \r\n");
				nrf_delay_ms(50);
				APP_ERROR_CHECK(err_code);			
				// fds_test_find_and_delete();			ta
				DefaultSettings = false;
			}	
		}
		//!---------------------------------------------
		err_code = sd_ble_gap_addr_get(&ble_addr);
		APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("46 - %d \r\n", err_code);
		//!Build advertising data struct to pass into @ref ble_advertising_init.
		memset(&advdata, 0, sizeof(advdata));
		advdata.name_type               =  BLE_ADVDATA_NO_NAME;
		advdata.include_appearance      = false;
		advdata.flags                   = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
		advdata.uuids_complete.uuid_cnt = sizeof(m_adv_uuids) / sizeof(m_adv_uuids[0]);
		advdata.uuids_complete.p_uuids  = m_adv_uuids;
		
			//!Configuration of manufacturer specific data
		adv_manuf_data_data[0] = 0x52;
    adv_manuf_data_data[1] = 0x42;
		adv_manuf_data_data[2] = 0x04;
    adv_manuf_data_data[3] = 0x00;
    adv_manuf_data_data[4] = 0x04;													 
		adv_manuf_data_data[5] = treal;
		adv_manuf_data_data[6] = tpoint;
		adv_manuf_data_data[7] = battery;
		adv_manuf_data_data[8] = data;
		adv_manuf_data_data[9] = h_lsb;
		adv_manuf_data_data[10] = h_msb;
		adv_manuf_data_data[11] = p_msb;
		adv_manuf_data_data[12] = p_lsb;
		adv_manuf_data_data[13] = 0x00;
		adv_manuf_data_data[14] = 0x3E;
		adv_manuf_data_array.p_data = adv_manuf_data_data;	
		adv_manuf_data_array.size = 15;
		adv_manuf_data.company_identifier = 0x4252;
		adv_manuf_data.data = adv_manuf_data_array;
		advdata.p_manuf_specific_data = &adv_manuf_data;

		err_code = ble_advdata_set(&advdata,NULL);
		APP_ERROR_CHECK(err_code);		
}

/**@snippet [UART Initialization] */

void in_light_pin_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
		INTLIGHT = true;
}
/**
 * @brief Function for configuring: PIN_IN pin for input, PIN_OUT pin for output,
 * and configures GPIOTE to give an interrupt on pin change.
 */
static void gpio_init(void)
{
		ret_code_t err_code;
		err_code = nrf_drv_gpiote_init();
		APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("48 - %d \r\n", err_code);
		nrf_drv_gpiote_out_config_t out_config = GPIOTE_CONFIG_OUT_SIMPLE(false);
		err_code = nrf_drv_gpiote_out_init(ARDUINO_6_PIN, &out_config);   //OPT
		APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("49 - %d \r\n", err_code);
	  err_code = nrf_drv_gpiote_out_init(ARDUINO_8_PIN, &out_config);   //OPT
		APP_ERROR_CHECK(err_code);
	  err_code = nrf_drv_gpiote_out_init(ARDUINO_A4_PIN, &out_config);   //BMI
		APP_ERROR_CHECK(err_code);
	  err_code = nrf_drv_gpiote_out_init(ARDUINO_5_PIN, &out_config);  //BME
    APP_ERROR_CHECK(err_code);
	  nrf_gpio_cfg_input(30, NRF_GPIO_PIN_NOPULL);
		nrf_gpio_input_disconnect(30);
		nrf_gpio_cfg_input(7, NRF_GPIO_PIN_NOPULL);
		nrf_gpio_input_disconnect(7);
		nrf_gpio_cfg_input(31, NRF_GPIO_PIN_NOPULL);
		nrf_gpio_input_disconnect(31);
		nrf_gpio_cfg_input(29, NRF_GPIO_PIN_NOPULL);
		nrf_gpio_input_disconnect(29);
		nrf_gpio_cfg_input(17, NRF_GPIO_PIN_NOPULL);
		nrf_gpio_input_disconnect(17);
		
		
}

void cal_count( int prf)
{
	count_week = (40/prf)*60*24*7;//*24;//(604800/prf);
}

int main(void)
{
		uint32_t err_code;
		bool     erase_bonds = true;
		//!Initialize.
		err_code = NRF_LOG_INIT(NULL);
	  NRF_LOG_INFO("F%d \r\n", err_code);
		APP_ERROR_CHECK(err_code);
	  //NRF_LOG_INFO("51 - %d \r\n", err_code);
	  NRF_LOG_INFO("Start V1\r\n");
		//!Initialize the application timer module.
		APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, false);
		//!Create application timers.
		create_timers();
		twi_init();
		gpio_init();
	
    nrf_gpio_pin_write(ARDUINO_6_PIN, 1);  //opt1
		nrf_gpio_pin_write(ARDUINO_8_PIN, 1); //opt2
	//	nrf_gpio_pin_write(ARDUINO_5_PIN, 1); //bme
	
	 calibratebme280();
	 
	
		fls_count();
		cal_weeks(((VALUEOFF)),10);
		//cal_count(30);
		cal_count(((VALUEOFF)));
		buttons_leds_init(/*&erase_bonds*/);
		ble_stack_init();
		err_code = radio_notification_init(3, NRF_RADIO_NOTIFICATION_TYPE_INT_ON_BOTH, NRF_RADIO_NOTIFICATION_DISTANCE_800US);
		APP_ERROR_CHECK(err_code);
		//NRF_LOG_INFO("52 - %d \r\n", err_code);
		peer_manager_init(erase_bonds);
		
		if (erase_bonds == true)
		{
			NRF_LOG_DEBUG("Bonds erased!\r\n");
		}
		gap_params_init();
		advertising_init(false);
		sd_power_dcdc_mode_set(1);
		services_init();
		NRF_LOG_INFO("Services are initialized\r\n");
		conn_params_init();
		NRF_LOG_INFO("conn params are initialized\r\n");
		//!Start execution.
		err_code =fds_test_init();
		APP_ERROR_CHECK(err_code);
		NRF_LOG_INFO("53 - %d \r\n", err_code);
		read_flash_data();
		read_flash_data_fds();
		read_flash_data_conf();
		NRF_LOG_INFO("SystemOn\r\n");
		if (SystemOn == true)
		{
			advertising_start(channel_37,channel_38,channel_39);
			bsp_board_led_on(BSP_BOARD_LED_0);
			nrf_delay_ms(100);
			bsp_board_led_off(BSP_BOARD_LED_0);
		}
		NRF_LOG_INFO("St\r\n");
		//!Enter main loop.
		for (;;)
		{
			if (NRF_LOG_PROCESS() == false)
			{
				if(m_update_adv_pkts)
				{
					update_adv_data();					
					m_update_adv_pkts = false;
				}
				power_manage();
			}
		}
}
/**
 * @}
 */
